/*
 * Implementation of class FileName
 *
 * $Date: 2003/09/17 15:46:48 $
 * $Revision: 1.4 $
 *
 */

#include "WImage/debug.h"
#include "WImage/FileName.hh"
#include <string.h>
#include <ctype.h>
#include <iostream>
#include <stdexcept>
#include <assert.h>

#ifndef __WATCOMC__
using namespace std;
#else
#include <string>
#endif

FileName::FileName (char const *name)
{
  m_isdos = false;
  
  if (name == NULL)
    {
      throw invalid_argument ("FileName: NULL reference to filename.");
    }
  char *dos = strrchr (name, '\\');
  char *unx = strrchr (name, '/');
  char const  *extn = name;
  char const *base = name;
  int size;
  
  size = strlen (name) + 1;
  m_fname = new char[size];
  strcpy (m_fname, name);

  if (dos != NULL)
    {
      m_isdos = true;
    }
  
  if (dos != NULL || unx != NULL)
    {
      base = dos > unx? dos: unx;
    }
  
  base++;
  size -= strlen (base);
  m_dirname = new char[size];
  strncpy (m_dirname, name, size);
  m_dirname[size-1] = '\0';


  extn = strrchr (base, '.');

  if (extn == NULL)
    {
      m_extension = new char[1];
      m_extension[0] = '\0';
      m_basename=new char[strlen(base)+1];
      strcpy(m_basename, base);
    }
  else
    {
      m_extension = new char[strlen (extn) + 1];
      strcpy (m_extension, extn);

      size = extn - base + 1;
      assert(extn + 1 > base);
      m_basename = new char[size];
      strncpy (m_basename, base, size);
      m_basename[size-1] = '\0';
    }

  guessext ();
}

FileName::~FileName (void)
{
  delete [] m_fname;
  delete [] m_dirname;
  delete [] m_basename;
  delete [] m_extension;
}

void
FileName::guessext (void)
{
  if (isext (".pgm"))
    {
      m_ftype = fn_pgm;
    }
  else if (isext (".raw"))
    {
      m_ftype = fn_raw;
    }
#ifdef PFI
  else if (isext (".pfi"))
    {
      m_ftype = fn_pfi;
    }
#endif
  else if (isext (".ppm"))
    {
      m_ftype = fn_ppm;
    }
  else if (isext (".jpg") || isext (".jpeg"))
    {
      m_ftype = fn_jpg;
    }
  else
    {
      m_ftype = fn_unknown;
    }
}

bool
FileName::isext (const char *ext) const
{
  int i;
  for (i = 0; ext[i] != '\0' && m_extension[i] != '\0'; i++)
    {
      if (tolower ((unsigned char)ext[i]) 
          != (tolower ((unsigned char)m_extension[i])))
        {
          return false;
        }
    }
  return ext[i] == '\0' && m_extension[i] == '\0';
}
  
void 
FileName::toext (const char *ext)
{
  delete [] m_extension;
  m_extension = new char[strlen (ext) + 1];
  strcpy (m_extension, ext);
  guessext ();
}

