/*
 * Wavelets on Greyscale images (PGM)
 *
 * $Date: 2004/05/13 17:51:19 $
 * $Revision: 1.5 $
 *
 */

#include <iostream>
#include <stdexcept>
#include <assert.h>
#include <stdio.h>
#include <math.h>
#include <Wave.hh>

#ifndef __WATCOMC__
using namespace std;
#endif

int
main (int argc, char *argv[])
{
  int stages = 1;
  FilterSet *flt = &Daub4;
  
  cout << "wavepgm version " << WAVE_VERSION << " (c) 2001-2003 by Herbert"
#ifdef PFI
       << " (pfi support enabled)"
#endif
       << endl;

  bool standard = false;

  if (argc > 2 && strcmp (argv[1], "-s") == 0)
    {
      standard = true;
      argc--;
      argv++;
    }

  if (argc < 3 || argc > 8)
    {
      cerr << "Usage: wavepgm [-s] <infile> <transformed> [<optional>]\n"
           << "      optional: "
           << "[stages [filter [<reconstructed> [<beautified>]]]]\n";
      return 1;
    }
  
  if (argc > 3)
    {
      if (sscanf (argv[3], "%d", &stages) != 1)
        {
          cerr << "Invalid stages: " << argv[3] << endl;
          return 1;
        }
    }
  if (argc > 4)
    {
      try
        {
          flt = &FilterSet::filterFromString (argv[4]);
        }
      catch (const exception &error)
        {
          cerr << "Error, : " << error.what () << endl;
          return 1;
        }
    }
  
  cout << "initialize:\n";
  
  Image *img = new StillImage ();
  Image *beautified = NULL;
  
  cout << "read image " << argv[1] << "...\n";

  try
    {
      img->read (argv[1]);
    }
  catch (const exception &error)
    {
      cerr << "Reading image: " << error.what () <<endl;
      return 1;
    }

  WaveletTransform *transform;

  if (standard)
    {
      transform = new StandardTransform (*img, *flt);
    }
  else
    {
      transform = new PyramidTransform (*img, *flt);
    }
  
  cout << "analysis...\n";
  
  try
    {
      transform->analysis (stages);
    }
  catch (const exception &error)
    {
      cerr << "Analysis: " << error.what () <<endl;
      return 1;
    }

  cout << "write transformed image " << argv[2] << "...\n";
  try
    {
      img->write (argv[2]);
    }
  catch (const exception &error)
    {
      cerr << "Writing transformed image: " << error.what () << endl;
      return 1;
    }
  
  if (argc > 5)
    {

      if (argc > 6)
        {
          beautified = img->clone ();
        }
      
      cout << "synthesis...\n";
      
      try
        {
          transform->synthesis (stages, stages);
        }
      catch (const exception &error)
        {
          cerr << "Synthesis: " << error.what () <<endl;
          return 1;
        }

      cout << "write reconstructed image " << argv[5] << "...\n";

      try
        {
          img->write (argv[5]);
        }
      catch (const exception &error)
        {
          cerr << "Writing reconstructed image: " << error.what () << endl;
          return 1;
        }
    }

  if (argc > 6)
    {
      cout << "beautification...\n";
      int y, x, a, b;
      transform->where (LL, stages, y, x, a, b);
      beautified->beautify (a, b, y, x);
      
      cout << "write beautified image " << argv[6] << "...\n";

      try
        {
          beautified->write (argv[6]);
        }
      catch (const exception &error)
        {
          cerr << "Writing beautified image: " << error.what () << endl;
          return 1;
        }

      delete beautified;
    }

  cout << "done.\n";
  
  delete img;
  delete transform;
  
  return 0;
}

