/*
 * Wavelets on Color images (PPM)
 *
 * $Date: 2004/05/13 17:51:19 $
 * $Revision: 1.7 $
 *
 */

#include <iostream>
#include <stdexcept>
#include <assert.h>
#include <stdio.h>
#include <math.h>
#include <Wave.hh>

#ifndef __WATCOMC__
using namespace std;
#endif

int
main (int argc, char *argv[])
{
  int stages = 1;
  FilterSet *flt = &Daub4;
  
  cout << "waveppm version " << WAVE_VERSION << " (c) 2001-2003 by Herbert"
#ifdef PFI
       << " (pfi support enabled)"
#endif
       << endl;

  bool standard = false;

  if (argc > 2 && strcmp (argv[1], "-s") == 0)
    {
      standard = true;
      argc--;
      argv++;
    }

  if (argc < 3 || argc > 8)
    {
      cerr << "Usage: [-s] waveppm <infile> <transformed> [<optional>]\n"
           << "      optional: "
           << "[stages [filter [<reconstructed> [<beautified>]]]]\n";
      return 1;
    }
  
  if (argc > 3)
    {
      if (sscanf (argv[3], "%d", &stages) != 1)
        {
          cerr << "Invalid stages: " << argv[3] << endl;
          return 1;
        }
    }
  if (argc > 4)
    {
      try
        {
          flt = &FilterSet::filterFromString (argv[4]);
        }
      catch (const exception &error)
        {
          cerr << "Error, : " << error.what () << endl;
          return 1;
        }
    }
  
  cout << "initialize:\n";
  
  ColorImage *ppm = new ColorImage ();

  cout << "read image " << argv[1] << "...\n";

  try
    {
      ppm->read (argv[1]);
    }
  catch (const exception &error)
    {
      cerr << "Reading image: " << error.what () <<endl;
      return 1;
    }
  cout << "done.\n";

  WaveletTransform **transforms = new WaveletTransform* [ppm->colors ()];
  ColorImage *beautified = NULL;

  for (int i = 0; i < ppm->colors (); i++)
    {
      if (standard)
        {
          transforms[i] = new StandardTransform (ppm->channel (i), *flt);
        }
      else
        {
          transforms[i] = new PyramidTransform (ppm->channel (i), *flt);
        }
    }
  
  cout << "analysis...\n";
  
  for (int i = 0; i < ppm->colors (); i++)
    {
      try
        {
          transforms[i]->analysis (stages);
        }
      catch (const exception &error)
        {
          cerr << "Analysis: " << error.what () <<endl;
          return 1;
        }
    }

  cout << "write transformed image " << argv[2] << "...\n";
  try
    {
      ppm->write (argv[2]);
    }
  catch (const exception &error)
    {
      cerr << "Writing transformed image: " << error.what () << endl;
      return 1;
    }

  if (argc > 5)
    {

      if (argc > 6)
        {
          beautified = ppm->clone ();
        }

      cout << "synthesis...\n";

      for (int i = 0; i < ppm->colors (); i++)
        {
          try
            {
              transforms[i]->synthesis (stages);
            }
          catch (const exception &error)
            {
              cerr << "Synthesis: " << error.what () <<endl;
              return 1;
            }
        }

      cout << "write reconstructed image " << argv[5] << "...\n";

      try
        {
          ppm->write (argv[5]);
        }
      catch (const exception &error)
        {
          cerr << "Writing reconstructed image: " << error.what () << endl;
          return 1;
        }
    }

  if (argc > 6)
    {
      cout << "beautification...\n";
      int y, x, a, b;
      transforms[0]->where (LL, stages, y, x, a, b);

      for (int i = 0; i < ppm->colors (); i++)
        {
          (beautified->channel (i)).beautify (a, b, y, x);
        }
      
      cout << "write beautified image " << argv[6] << "...\n";

      try
        {
          beautified->write (argv[6]);
        }
      catch (const exception &error)
        {
          cerr << "Writing beautified image: " << error.what () << endl;
          return 1;
        }

      delete ppm;
      delete beautified;
    }

  cout << "done.\n";
  
  for (int i = 0; i < ppm->colors (); i++)
    {
      delete transforms[i];
    }
  delete [] transforms;
  
  return 0;
}

